<?php

namespace Drupal\webform_headless\Controller;

use Drupal\Component\Plugin\Exception\PluginException;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\webform\WebformInterface;
use Drupal\webform_headless\Exception\WebformAccessDeniedException;
use Drupal\webform_headless\WebformJsonSchemaManager;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

/**
 * Controller for getting the schema.
 */
class GetSchemaController implements ContainerInjectionInterface {

  /**
   * The plugin manager.
   */
  protected readonly WebformJsonSchemaManager $jsonSchemaManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    $instance = new static();
    $instance->jsonSchemaManager = $container->get('plugin.manager.webform_headless');

    return $instance;
  }

  /**
   * Gets the schema.
   */
  public function __invoke(WebformInterface $webform, Request $request): JsonResponse {
    $format = $request->query->get('schema');

    try {
      $schema = $this->jsonSchemaManager->createInstance($format);
    }
    catch (PluginException $e) {
      return new JsonResponse(
        [
          'errors' => [
            ['message' => sprintf("The schema format '%s' is not supported.", $format)],
          ],
        ],
        Response::HTTP_BAD_REQUEST,
      );
    }

    try {
      $data = $schema->toArray($webform, $request);
    }
    catch (WebformAccessDeniedException $e) {
      return new JsonResponse(
        [
          'errors' => array_map(
            static fn (string $error) => ['message' => $error],
            $e->getErrors(),
          ),
        ],
        Response::HTTP_FORBIDDEN,
      );
    }

    return new JsonResponse($data);
  }

}
