<?php

namespace Drupal\webform_headless\Controller;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\webform\WebformInterface;
use Drupal\webform\WebformSubmissionInterface;
use Drupal\webform_headless\Submitter;
use Drupal\webform_headless\WebformJsonSchemaManager;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Controller for getting a submission.
 */
class GetSubmissionController implements ContainerInjectionInterface {

  /**
   * The submitter.
   */
  protected readonly Submitter $submitter;

  /**
   * The plugin manager.
   */
  protected readonly WebformJsonSchemaManager $jsonSchemaManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    $instance = new static();
    $instance->submitter = $container->get('webform_headless.submitter');
    $instance->jsonSchemaManager = $container->get('plugin.manager.webform_headless');

    return $instance;
  }

  /**
   * Gets the submission.
   */
  public function __invoke(WebformInterface $webform, WebformSubmissionInterface $submission, Request $request): ?JsonResponse {
    if ($submission->getWebform()?->id() !== $webform->id()) {
      throw new NotFoundHttpException();
    }

    $format = $request->query->get('schema');
    $schema = $this->jsonSchemaManager->createInstance($format);

    return $this->submitter->getSubmission($submission, $schema);
  }

}
