<?php

namespace Drupal\webform_headless\Controller;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\webform\WebformSubmissionInterface;
use Drupal\webform_headless\Submitter;
use Drupal\webform_headless\WebformJsonSchemaManager;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

/**
 * Controller for updating a submission.
 */
class UpdateSubmissionController implements ContainerInjectionInterface {

  /**
   * The submitter.
   */
  protected readonly Submitter $submitter;

  /**
   * The plugin manager.
   */
  protected readonly WebformJsonSchemaManager $jsonSchemaManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    $instance = new static();
    $instance->submitter = $container->get('webform_headless.submitter');
    $instance->jsonSchemaManager = $container->get('plugin.manager.webform_headless');

    return $instance;
  }

  /**
   * Updates a submission.
   */
  public function __invoke(WebformSubmissionInterface $submission, Request $request): ?JsonResponse {
    $format = $request->query->get('schema');
    $schema = $this->jsonSchemaManager->createInstance($format);

    try {
      $data = json_decode($request->getContent(), TRUE, 512, JSON_THROW_ON_ERROR);
    }
    catch (\JsonException $exception) {
      return new JsonResponse(
        ['errors' => [['message' => sprintf('Error while parsing json: %s', $exception->getMessage())]]],
        Response::HTTP_BAD_REQUEST
      );
    }

    return $this->submitter->updateSubmission($submission, $schema, $data);
  }

}
